"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class FirFilter {
    constructor(coeffs) {
        this.ptr = 0;
        this.coeffs = coeffs;
        // buffer actually is a window
        this.buffer = new Float32Array(this._order);
        this.reset();
    }
    get _order() {
        return this.coeffs.length;
    }
    /**
     * @param  {number[]} input
     * @returns number[]
     */
    filter(input) {
        const output = new Float32Array(input.length);
        for (let i = 0; i < input.length; ++i) {
            this.buffer[this.ptr] = input[i];
            // start from oldest to newest, sum from 0 to 10 equals to 10 to 0
            for (let j = 1; j <= this._order; ++j) {
                // idx actually point to the oldest value involed
                let idx = this.ptr + j;
                if (idx >= this._order) {
                    idx -= this._order;
                }
                output[i] += this.buffer[idx] * this.coeffs[j - 1];
            }
            ++this.ptr;
            if (this.ptr >= this._order) {
                this.ptr -= this._order;
            }
        }
        return [].slice.call(output);
    }
    /**
     * call this method if you want to filter another serials of data
     */
    reset() {
        this.ptr = 0;
        for (let i = 0; i < this.buffer.length; ++i) {
            this.buffer[i] = 0;
        }
    }
}
exports.default = FirFilter;
//# sourceMappingURL=index.js.map